function filterBank = createFilterBank()

gaussianSigmas = [1,2,4];
gaussianHSize = 2*(3*gaussianSigmas) + 1;

LoGSigmas = [1,2,4,8];
LoGHSize = 2*(3*LoGSigmas) + 1;

dGaussianSigmas = [2,4];
dGaussianHSize = 2*(3*dGaussianSigmas) + 1;

filterBank = cell(1,3);

%Guassians (Apply to CIE L,a,b channels)
gaussians = cell(1,size(gaussianSigmas,2));
for i = 1:size(gaussianSigmas,2)
    gaussians{i} = fspecial('gaussian',gaussianHSize(i),gaussianSigmas(i));
end
filterBank{1} = gaussians;

%Laplacian of Gaussians (Apply to CIE L channel)
LoGs = cell(1,size(LoGSigmas,2));
for i = 1:size(LoGSigmas,2)
    LoGs{i} = fspecial('log',LoGHSize(i),LoGSigmas(i));
end
filterBank{2} = LoGs;

%Firt Order Derivatives of Guassians in x and y directions
%(Apply to CIE L channel)
dGaussians = cell(1,2*size(dGaussianSigmas,2));
for i = 1:size(dGaussianSigmas,2)
    dGaussians{i} = discretizeFilter(@derivOfGaussian,dGaussianHSize(i),dGaussianSigmas(i),1);
    dGaussians{i+size(dGaussianSigmas,2)} = discretizeFilter(@derivOfGaussian,dGaussianHSize(i),dGaussianSigmas(i),2);
end
filterBank{3} = dGaussians;


function B = discretizeFilter(aFunc,HSize,sigma,dVar)
%dVar: 1:(deriv wrt x) 2:(deriv wrt y)
B = zeros(HSize);
for i = 1:HSize
    x = i - (HSize-1)/2 - 1;
    for j = 1:HSize
        y = (j - (HSize-1)/2 - 1);
        if dVar == 1
            var = x;
        else
            var = y;
        end
        B(j,i) = aFunc(x,y,sigma,var);
    end
end

function g = gaussian(x,y,sigma)
g = 1/(2*pi*sigma^2)*exp(-(x^2+y^2)/(2*sigma^2));

function dg = derivOfGaussian(x,y,sigma,var)
%var: x:(dG/dx); y:(dG/dy)
dg = -var/(2*pi*sigma^4)*exp(-(x^2+y^2)/(2*sigma^2));

function log = LapOfGaussian(x,y,sigma)
log = (x^2+y^2)/(2*pi*sigma^6)*exp(-(x^2+y^2)/(2*sigma^2));

